#!/bin/bash

umask 002

#nodes=${1:-"0 1 2 3"}
nodes=${1:-"0"}

if [[ -z $Path_SICw ]]; then
  if ( [ -f /proc/sicread ] && [ -f /proc/sicregs ] ); then
    Path_SICw=/proc/sicread
    Path_SICr=/proc/sicregs
  elif  ( [ -f /proc/sicread ] && [ -f /proc/sicwrite ] ); then
    Path_SICw=/proc/sicread
    Path_SICr=/proc/sicread
  fi
  [[ ! -f $Path_SICw ]] && exit 2
fi

READ_ATTEMPT_NUMBER=20
get_reg() {
    local r
    local s
    local a
    local N=0
    local hex1=$(printf "0x%x" $1)
    while true; do
	((N++))
	echo ${hex1} > $Path_SICw
	s=`cat ${Path_SICr} | grep "node: $2 reg"`
	a=(`echo ${s,,} | awk '{print $4}' | tr '():' ' '`)
	if [[ $hex1 = ${a[0]} ]]; then
	    break
	fi
	if [[ -z ${a[0]} ]]; then
	    break
	fi
	if ((N>${READ_ATTEMPT_NUMBER})); then
	    break
	fi
	sleep .1
    done
    echo ${s}
}

#echo "0x0704" > $Path_SICw
#status_links[0]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[1]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[2]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[3]=`cat $Path_SICr| grep "node: 3 reg"`
status_links[0]=`get_reg "0x704" 0`
status_links[1]=`get_reg "0x704" 1`
status_links[2]=`get_reg "0x704" 2`
status_links[3]=`get_reg "0x704" 3`

#echo "0x0708" > $Path_SICw
#mode_links[0]=`cat $Path_SICr | grep "node: 0 reg"`
#mode_links[1]=`cat $Path_SICr | grep "node: 1 reg"`
#mode_links[2]=`cat $Path_SICr | grep "node: 2 reg"`
#mode_links[3]=`cat $Path_SICr | grep "node: 3 reg"`
mode_links[0]=`get_reg "0x708" 0`
mode_links[1]=`get_reg "0x708" 1`
mode_links[2]=`get_reg "0x708" 2`
mode_links[3]=`get_reg "0x708" 3`

err=0
for cpu in `echo ${nodes}`; do
	[[ ${cpu} == "0" ]] || [[ ${cpu} == "1" ]] || [[ ${cpu} == "2" ]] || [[ ${cpu} == "3" ]] || { echo "Error! Wrong NODE number=${cpu}"; exit 1; }

	warn=0
	if [ -z "${status_links[${cpu}]}" ]; then
		echo "Error! Cannot read status for io-link on NODE_${cpu}";
		err=1
		continue;
	fi
	if [ -z "${mode_links[${cpu}]}" ]; then
		echo "Error! Cannot read mode for io-link on NODE_${cpu}";
		err=1
		continue;
	fi
	status_links_pls=`echo "${status_links[${cpu}]}" | awk '{print $5}'`
	pls_cmd_act=$(( (${status_links_pls} & 0x00000080) >> 7 ))
	pls_link_state=$(( (${status_links_pls} & 0x80000000) >> 31 ))
	pls_ltssm_state=`printf "0x%x" $(( (${status_links_pls} & 0x07000000) >> 24 ))`
	pls_link_scale=`printf "0x%x" $(( (${status_links_pls} & 0x00000F00) >> 8 ))`
	pls_link_width=`printf "0x%x" $(( (${status_links_pls} & 0x000F0000) >> 16 ))`

	mode_links_plm=`echo "${mode_links[${cpu}]}" | awk '{print $5}'`
	plm_data_rate=$(( (${mode_links_plm} & 0x20000000) >> 29 ))


	if [ "${pls_link_scale}" != "0xf" ]; then
		echo "Error! Wrong status for io-link on NODE_${cpu}: not full scale=${pls_link_scale}/0xf (PLS=${status_links_pls})";
		err=1
		continue;
	fi

	if [ "${pls_cmd_act}" == "1" ]; then
		echo "Error! Wrong status for io-link on NODE_${cpu}: command executing (PLS=${status_links_pls})";
		err=1
		continue;
	fi
	if [ "${pls_link_state}" != "1" ]; then
		echo "Warning! Wrong status for io-link on NODE_${cpu}: link is not active (PLS=${status_links_pls})";
		warn=1
		err=1
	fi
	if [ "${pls_ltssm_state}" != "0x3" ]; then
		echo "Warning! Wrong status for io-link on NODE_${cpu}: ltssm is not linkup (PLS=${status_links_pls})";
		warn=1
		err=1
	fi
	if [ "${plm_data_rate}" != "1" ]; then
		echo "Warning! Wrong mode for io-link on NODE_${cpu}: low data rate (PLS=${status_links_pls}, PLM=${mode_links_plm})";
		warn=1
		err=1
	fi
	if [ "${pls_link_width}" != "0xf" ]; then
		echo "Warning! Wrong status for io-link on NODE_${cpu}: not full width=${pls_link_width}/0xf (PLS=${status_links_pls})";
		warn=1
		err=1
		wdth=`echo ${pls_link_width} | sed s/0x//`
		cl=""
		[[ "$(( (${wdth} & 0x1) >> 0 ))" == "1" ]] || cl="$cl 0 1 2 3"
		[[ "$(( (${wdth} & 0x2) >> 1 ))" == "1" ]] || cl="$cl 4 5 6 7"
		[[ "$(( (${wdth} & 0x4) >> 2 ))" == "1" ]] || cl="$cl 8 9 10 11"
		[[ "$(( (${wdth} & 0x8) >> 3 ))" == "1" ]] || cl="$cl 12 13 14 15"
		echo "       advice: try to check lanes${cl} on io-link CPU_${cpu} <-> IOHUB_${cpu}";
	fi

	[[ "${warn}" != "1" ]] && echo "OK! Good status for io-link on NODE_${cpu} (PLS=${status_links_pls}, PLM=${mode_links_plm})";
done

[[ "${err}" == "1" ]] && exit 1
exit 0
